% Intrinsic and Extrinsic Camera Parameters
%
% This script file can be directly excecuted under Matlab to recover the camera intrinsic and extrinsic parameters.
% IMPORTANT: This file contains neither the structure of the calibration objects nor the image coordinates of the calibration points.
%            All those complementary variables are saved in the complete matlab data file Calib_Results.mat.
% For more information regarding the calibration model visit http://www.vision.caltech.edu/bouguetj/calib_doc/


%-- Focal length:
fc = [ 1051.442008501288700 ; 1049.134408107230000 ];

%-- Principal point:
cc = [ 499.341209983130280 ; 403.275637960574500 ];

%-- Skew coefficient:
alpha_c = 0.000000000000000;

%-- Distortion coefficients:
kc = [ -0.185733155864108 ; -0.103534144017362 ; -0.000705469488472 ; -0.002708449360881 ; 0.000000000000000 ];

%-- Focal length uncertainty:
fc_error = [ 24.082636891999698 ; 24.475695720676505 ];

%-- Principal point uncertainty:
cc_error = [ 11.708224066367487 ; 10.453739419688334 ];

%-- Skew coefficient uncertainty:
alpha_c_error = 0.000000000000000;

%-- Distortion coefficients uncertainty:
kc_error = [ 0.074150851378818 ; 0.871042857392535 ; 0.001897684206679 ; 0.002167797838544 ; 0.000000000000000 ];

%-- Image size:
nx = 1024;
ny = 768;


%-- Various other variables (may be ignored if you do not use the Matlab Calibration Toolbox):
%-- Those variables are used to control which intrinsic parameters should be optimized

n_ima = 2;						% Number of calibration images
est_fc = [ 1 ; 1 ];					% Estimation indicator of the two focal variables
est_aspect_ratio = 1;				% Estimation indicator of the aspect ratio fc(2)/fc(1)
center_optim = 1;					% Estimation indicator of the principal point
est_alpha = 0;						% Estimation indicator of the skew coefficient
est_dist = [ 1 ; 1 ; 1 ; 1 ; 0 ];	% Estimation indicator of the distortion coefficients


%-- Extrinsic parameters:
%-- The rotation (omc_kk) and the translation (Tc_kk) vectors for every calibration image and their uncertainties

%-- Image #1:
omc_1 = [ -1.705877e-002 ; -3.075252e+000 ; 3.737950e-001 ];
Tc_1  = [ 1.081494e+002 ; -9.618872e+001 ; 5.049244e+002 ];
omc_error_1 = [ 2.438433e-003 ; 1.054491e-002 ; 1.615442e-002 ];
Tc_error_1  = [ 5.638284e+000 ; 5.049993e+000 ; 1.125181e+001 ];

%-- Image #2:
omc_2 = [ -2.019179e+000 ; -2.035754e+000 ; 2.749553e-001 ];
Tc_2  = [ -8.218073e+001 ; -1.071911e+002 ; 5.387945e+002 ];
omc_error_2 = [ 7.580055e-003 ; 6.606340e-003 ; 1.546153e-002 ];
Tc_error_2  = [ 5.992878e+000 ; 5.373688e+000 ; 1.255385e+001 ];

